package main

import (
	`math/big`
	`net`
	"net/netip"

	`subnetter/netsplit`
)

// subnetResult is only used for human/"pretty" printing.
type subnetResult netip.Prefix

// tableLegacy4 contains a spec for a class in the legacy "classed" IPv4 networking.
type tableLegacy4 struct {
	Class    string
	CIDR     string
	Start    string
	End      string
	NetStart netip.Addr   `render:"-"`
	NetEnd   netip.Addr   `render:"-"`
	NetCIDR  netip.Prefix `render:"-"`
}

// tableMask4 is used to hold string representation of netmask information.
type tableMask4 struct {
	Prefix    string
	Netmask   string
	Hex       string
	Dec       uint32
	Bin       string
	Mask      net.IPMask   `render:"-"`
	NetPrefix netip.Prefix `render:"-"`
}

// tablePrefixNote is used to hold notes about select prefix sizes.
type tablePrefixNote struct {
	Prefix    string
	Note      string
	NetPrefix netip.Prefix `render:"-"`
}

// tablePrefix is used to hold string representation of network prefixes.
type tablePrefix struct {
	Prefix    string
	Bits      uint8
	Addresses *big.Int
	Hosts     *big.Int
	NetPrefix netip.Prefix `render:"-"`
}

// tableFormatter is used for "rendering" table output.
type tableFormatter struct {
	// Headers...
	// First char, first line
	TopLeftHdr string
	// Char to fill between TopLeftHdr and TopColSepHdr.
	TopFillHdr string
	// Column separator, first line
	TopColSepHdr string
	// Last char, first line
	TopRightHdr string
	// Column separator for both column separators (title line) and left/right-most lines.
	ColSepHdr string
	// First char, last line of header
	BottomLeftHdr string
	// Char to fill between BottomLeftHdr and BottomColSepHdr.
	BottomFillHdr string
	// Column separator, last line of header
	BottomColSepHdr string
	// Last char, last line of header
	BottomRightHdr string
	// Rows...
	// Left-most line (border).
	Left string
	// Fill the cell lines for values
	Fill string
	// Separate line/cell border columns
	LineColSep string
	// "In-between" rows; left-most.
	LineLeft string
	// "In-between" rows, right-most.
	LineRight string
	// Separate value columns
	ColSep string
	// Right-most line (border)
	Right string
	// Last lines get special treatment as they are also a border.
	LastLeft  string
	LastFill  string
	LastSep   string
	LastRight string
	// This is mostly for experimentation for Plain output, but...
	SuppressLineSep bool
	NoUpperTitle    bool
	NoBoldTitle     bool
}

type ReservedResults struct {
	Opts     CheckArgs
	Reserved map[netip.Prefix]*netsplit.IANAAddrNetResRecord
}
